
create or replace PROCEDURE CreateTempTables
AS
BEGIN 

    BEGIN
        EXECUTE IMMEDIATE 'CREATE TABLE ASPStateTempSessions 
		(
		SessionId nvarchar2 (88) NOT NULL , 
		Created timestamp  DEFAULT  ( SYS_EXTRACT_UTC(SYSTIMESTAMP) ) NOT NULL , 
		Expires timestamp  NOT NULL , 
		LockDate timestamp  NOT NULL , 
		LockDateLocal timestamp  NOT NULL , 
		LockCookie int  NOT NULL , 
		Timeout int  NOT NULL , 
		Locked number (1) NOT NULL , 
		SessionItem blob , 
		Flags int  DEFAULT  ( 0 ) NOT NULL , 
		CONSTRAINT PK_ASPStateTempSessions PRIMARY KEY (SessionId) 
		) TABLESPACE "OSSES" CACHE NOLOGGING';
	END;
	
	BEGIN EXECUTE IMMEDIATE 'DROP INDEX OSIDX_ASPSESSIONS_SessIdCIAI TABLESPACE "OSSES"'; EXCEPTION WHEN OTHERS THEN NULL; END;
%#IF_CIAI%	BEGIN EXECUTE IMMEDIATE 'CREATE INDEX OSIDX_ASPSESSIONS_SessIdCIAI ON ASPStateTempSessions  ( NLSSORT (SessionId, ''nls_sort=''''BINARY_AI'''''') ) TABLESPACE "OSSES"'; END;%#ENDIF_CIAI%

	BEGIN 
		EXECUTE IMMEDIATE 'DROP INDEX OSIDX_ASPSESSIONS_Expire TABLESPACE "OSSES"';
		EXCEPTION WHEN OTHERS THEN NULL;
	END;
	
	
    BEGIN
        EXECUTE IMMEDIATE 'CREATE INDEX OSIDX_ASPSESSIONS_Expire ON ASPStateTempSessions  ( Expires ) TABLESPACE "OSSES"';
    END;
		
	
    BEGIN
        EXECUTE IMMEDIATE 'CREATE TABLE ASPStateTempApplications 
		(
		AppId int  NOT NULL , 
		AppName varchar2 (280) NOT NULL , 
		CONSTRAINT PK_ASPStateTempApps PRIMARY KEY (AppId) 
		) TABLESPACE "OSSES"';        
	END;
	
	
	BEGIN 
		EXECUTE IMMEDIATE 'DROP INDEX OSIDX_ASPTEMPAPPS_App TABLESPACE "OSSES"';
		EXCEPTION WHEN OTHERS THEN NULL;
	END;
	
	
    BEGIN 
		EXECUTE IMMEDIATE 'CREATE INDEX OSIDX_ASPTEMPAPPS_App ON ASPStateTempApplications  ( %#IF_CIAI%NLSSORT(%#ENDIF_CIAI%AppName%#IF_CIAI%, ''nls_sort=''''BINARY_AI'''''')%#ENDIF_CIAI% ) TABLESPACE "OSSES"';        
	END;
	
	
	BEGIN 
		EXECUTE IMMEDIATE 'CREATE TABLE ASPStateTempSessionsExtVars 
		(
		SessionId nvarchar2 (88) NOT NULL , 
		Variable varchar2 (100) NOT NULL , 
		Data blob ,
		UserId number (10,0) ,
		Cookie nvarchar2 (88) 
		) TABLESPACE "OSSES" CACHE';
	END;

	
	BEGIN 
		EXECUTE IMMEDIATE 'DROP INDEX OSIDX_ASPEXT_AppName TABLESPACE "OSSES"';
		EXCEPTION WHEN OTHERS THEN NULL;
	END;

	
	BEGIN
		EXECUTE IMMEDIATE 'CREATE INDEX OSIDX_ASPEXT_SessionId ON ASPStateTempSessionsExtVars  ( %#IF_CIAI%NLSSORT(%#ENDIF_CIAI%SessionId%#IF_CIAI%, ''nls_sort=''''BINARY_AI'''''')%#ENDIF_CIAI% ) TABLESPACE "OSSES"';
	END;
	

	BEGIN 
		EXECUTE IMMEDIATE 'DROP INDEX OSIDX_ASPEXT_AppNameVar TABLESPACE "OSSES"';
		EXCEPTION WHEN OTHERS THEN NULL;
	END;

	
	BEGIN
		EXECUTE IMMEDIATE 'CREATE INDEX OSIDX_ASPEXT_Variable ON ASPStateTempSessionsExtVars  ( %#IF_CIAI%NLSSORT(%#ENDIF_CIAI%SessionId%#IF_CIAI%, ''nls_sort=''''BINARY_AI'''''')%#ENDIF_CIAI%, %#IF_CIAI%NLSSORT(%#ENDIF_CIAI%Variable%#IF_CIAI%, ''nls_sort=''''BINARY_AI'''''')%#ENDIF_CIAI% ) TABLESPACE "OSSES"';
	END;

      
END;
/


create or replace PROCEDURE DropTempTables
AS
BEGIN 
    BEGIN
        EXECUTE IMMEDIATE 'DROP TABLE ASPStateTempSessions';
        EXCEPTION WHEN OTHERS THEN NULL;
	END;
	
    BEGIN
        EXECUTE IMMEDIATE 'DROP TABLE ASPStateTempApplications';
        EXCEPTION WHEN OTHERS THEN NULL;
	END;
	
	BEGIN
        EXECUTE IMMEDIATE 'DROP TABLE ASPStateTempSessionsExtVars';
        EXCEPTION WHEN OTHERS THEN NULL;
	END;
END;
/

call DROPTEMPTABLES()
/

call CREATETEMPTABLES()
/

create or replace function datediff( p_what in varchar2,
                                     p_d1   in date,
                                     p_d2   in date ) return number
   as
      l_result    number;
   begin
    select (p_d2-p_d1) *
              decode( upper(p_what),
                       'SS', 24*60*60, 'MI', 24*60, 'HH', 24, NULL )
    into l_result from dual;
 
    return l_result;
 end;
/


create or replace PROCEDURE DeleteExpiredSessions
AS
	now                        timestamp;
BEGIN  	
	now  :=  SYS_EXTRACT_UTC(SYSTIMESTAMP);
	
	DELETE FROM ASPStateTempSessionsExtVars
	WHERE SessionId IN (SELECT SessionId FROM ASPStateTempSessions WHERE Expires < DeleteExpiredSessions.now);
					   
	DELETE FROM ASPStateTempSessions   
	WHERE  Expires  < DeleteExpiredSessions.now;
END;
/

create or replace PROCEDURE DeleteExpiredSessionsIncr
(
deletedRowsCount	OUT	int
)
AS
	now                        timestamp;
BEGIN  	
	deletedRowsCount := 0;
	now  :=  SYS_EXTRACT_UTC(SYSTIMESTAMP);

	DELETE FROM ASPStateTempSessions   
	WHERE  Expires < DeleteExpiredSessionsIncr.now and rownum < 100
	RETURNING count(*) into deletedRowsCount;
  
	DELETE FROM ASPStateTempSessionsExtVars extVars
	WHERE NOT exists (SELECT * FROM ASPStateTempSessions tempSessions where extVars.sessionid=tempSessions.sessionid ) and rownum<100;
END;
/

create or replace PROCEDURE ResetData
AS
BEGIN
  DROPTEMPTABLES();
  
  CREATETEMPTABLES();
 
  RETURN;
END;
/



create or replace PROCEDURE TempGetAppID
(
	appName                  IN  varchar2  ,
	appNameHash              IN  int,
	appId                   OUT 	int
)
AS
	dupApp                     varchar2(280);
        appNameLower               varchar2(280);
        countcheck int;
              
BEGIN  	
	appNameLower  :=  LOWER(appName); 	               
        appId  := NULL;

        FOR REC IN ( 
            SELECT AppId FROM  ASPStateTempApplications 
            WHERE AppName  = appNameLower
	)
	LOOP
            appId := REC.AppId;
	END LOOP;
 
                	
	IF appId IS NULL THEN 	
		SAVEPOINT tran1;
                
                LOCK TABLE ASPStateTempApplications IN EXCLUSIVE MODE;
                
		FOR REC IN ( 
			SELECT AppId FROM  ASPStateTempApplications 
			WHERE AppName  = appNameLower
		)
		LOOP
			appId := REC.AppId;
		END LOOP;

		IF appId IS NULL THEN  
			
			INSERT INTO ASPStateTempApplications    
			VALUES ( appNameHash, appNameLower ); 
                        
                        appID := appNameHash;                        
		END IF;
		
		COMMIT;
		
	END IF;
               
EXCEPTION 
  WHEN DUP_VAL_ON_INDEX THEN
      FOR REC IN ( 
              SELECT RTRIM(AppName) TEMP_ALIAS_0 FROM  ASPStateTempApplications 
              WHERE AppId  = appNameHash
      )
      LOOP
              dupApp := REC.TEMP_ALIAS_0;
      END LOOP;
      
      RAISE_APPLICATION_ERROR(-20001,  'Oracle session state fatal error: hash-code collision between applications ''' || dupApp || ''' and ''' || AppNameLower || '''. Please rename the 1st application to resolve the problem.' );
END;
/



create or replace PROCEDURE TEMPGETSTATEITEM 
(
        id                          IN       	   nvarchar2 DEFAULT NULL,
        item                	      OUT      	   NOCOPY blob,
        locked                     OUT          NUMBER,
        lockAge                    OUT          int,
        lockCookie                 OUT          int,
        actionFlags                OUT          int
)
AS
        now        timestamp;
BEGIN 
        now  :=  SYS_EXTRACT_UTC(SYSTIMESTAMP); 
        -- retreive info
        SELECT Decode (ASPStateTempSessions.Locked, 0, SessionItem, 1, NULL),
               ASPStateTempSessions.Locked,                
               ASPStateTempSessions.LockCookie,
               Decode (ASPStateTempSessions.Locked, 0, 0, 1, DateDiff('SS', LockDate, now)),
               Flags
               INTO 
                 item, 
                 locked,                  
                 lockCookie,
                 lockAge,
                 actionFlags
        FROM ASPStateTempSessions
        WHERE SessionId = id AND now < Expires
		FOR UPDATE;
        
        -- update action flags                
        UPDATE ASPStateTempSessions
        SET 
            Flags = 0 -- always clear uninitialized flag with this call
        WHERE SessionId = id AND now < Expires;            
                  
		COMMIT;		
		
EXCEPTION
  WHEN NO_DATA_FOUND THEN NULL;  
END;
/



create or replace PROCEDURE TEMPGETSTATEITEMEXCLUSIVE 
(
        id                       IN      	nvarchar2 DEFAULT NULL,
        item                	   OUT      	NOCOPY blob,
        locked                  OUT          NUMBER,
        lockAge                 OUT          int,
        lockCookie              OUT          int,
        actionFlags             OUT          int
)
AS
        now        timestamp;
        nowLocal   timestamp;
BEGIN 
        
        now  :=  SYS_EXTRACT_UTC(SYSTIMESTAMP); 
        nowLocal := SYSDATE;
               
               --ASPStateTempSessions.LockCookie,  
        -- retreive info
        SELECT Decode (ASPStateTempSessions.Locked, 0, SessionItem, 1, NULL),
               ASPStateTempSessions.Locked, 
               Decode(ASPStateTempSessions.Locked, 0, LockCookie + 1, 1, LockCookie),               
               Flags,
               Decode (ASPStateTempSessions.Locked, 0, 0, 1, DateDiff('SS', LockDate, now))
               into 
                  item,
                  locked,
                  lockCookie,
                  actionFlags,
                  lockAge
        FROM ASPStateTempSessions
        WHERE SessionId = id AND now < Expires
		FOR UPDATE;
        
        -- update expiration date, lock status and action flags     
        UPDATE ASPStateTempSessions
        SET Expires = now + (1/24/60 * ASPStateTempSessions.Timeout),
            LockDate = Decode(Locked, 0, now, 1, LockDate),
            LockDateLocal = Decode(Locked, 0, nowLocal, 1, LockDateLocal),
            LockCookie = Decode(Locked, 0, LockCookie + 1, 1, LockCookie),
            Locked = 1,
            Flags = 0 -- always clear uninitialized flag with this call
        WHERE SessionId = id AND now < Expires;
		
		COMMIT;
		
EXCEPTION
  WHEN NO_DATA_FOUND THEN NULL;  
END;
/



create or replace PROCEDURE TempInsertStateItem
(
	id                       IN      	nvarchar2 DEFAULT NULL,
	item                	   IN      	blob DEFAULT NULL,
	timeout                  IN      	int DEFAULT NULL
)
AS
	now                        timestamp;
	nowLocal                   timestamp;
        expireDate                 timestamp;
BEGIN  	
	now  :=  SYS_EXTRACT_UTC(SYSTIMESTAMP); 	
	nowLocal  := (SYSDATE);
        expireDate := now + 1/24/60 * timeout; -- add timeout minutes to the date
	
	INSERT INTO ASPStateTempSessions ( 
                        SessionId , 
			SessionItem , 
			Timeout , 
			Expires , 
			Locked , 
			LockDate , 
			LockDateLocal , 
			LockCookie )  
	 VALUES 	(id , 
			item , 
			timeout,
			expireDate,
			0 , 
			now , 
			nowLocal , 
			1 ); 

END;
/


create or replace PROCEDURE TempInsertUninitializedItem
(
	id                       IN      	nvarchar2 DEFAULT NULL,
	item                	   IN      	blob DEFAULT NULL,
	timeout                  IN      	int DEFAULT NULL
)
AS
	now                        timestamp;
	nowLocal                   timestamp;
	expireDate                 timestamp;
BEGIN  	
	now  :=  SYS_EXTRACT_UTC(SYSTIMESTAMP); 	
	nowLocal  :=  (SYSDATE);
	expireDate := now + 1/24/60 * timeout; -- add timeout minutes to the date
	
	DELETE FROM ASPStateTempSessions WHERE SessionId = id;
	DELETE FROM ASPStateTempSessionsExtVars WHERE SessionId = id;

	INSERT INTO  ASPStateTempSessions   
			( SessionId , 
			SessionItem , 
			Timeout , 
			Expires , 
			Locked , 
			LockDate , 
			LockDateLocal , 
			LockCookie , 
			Flags )  
	 VALUES (id , 
			item , 
			timeout , 
			expireDate , 
			0 , 
			now , 
			nowLocal , 
			1 , 
			1 ); 
END;
/

create or replace PROCEDURE TempInsertInitializedItem
(
	id                       IN      	nvarchar2 DEFAULT NULL,
	item                	   IN      	blob DEFAULT NULL,
	timeout                  IN      	int DEFAULT NULL
)
AS
	now                        timestamp;
	nowLocal                   timestamp;
	expireDate                 timestamp;
BEGIN  	
	now  :=  SYS_EXTRACT_UTC(SYSTIMESTAMP); 	
	nowLocal  :=  (SYSDATE);
	expireDate := now + 1/24/60 * timeout; -- add timeout minutes to the date
	
	INSERT INTO  ASPStateTempSessions   
			( SessionId , 
			SessionItem , 
			Timeout , 
			Expires , 
			Locked , 
			LockDate , 
			LockDateLocal , 
			LockCookie , 
			Flags )  
	 VALUES (id , 
			item , 
			timeout , 
			expireDate , 
			1 , 
			now , 
			nowLocal , 
			1 , 
			0 ); 
END;
/

create or replace PROCEDURE TempReleaseStateItemExclusive
(
	id                       IN      	nvarchar2 DEFAULT NULL,
	lockCookie               IN      	int DEFAULT NULL
)
AS
        now timestamp;
BEGIN 

        now  :=  SYS_EXTRACT_UTC(SYSTIMESTAMP); 
        
	UPDATE  ASPStateTempSessions
	SET     Expires = now + (1/24/60 * ASPStateTempSessions.TIMEOUT),	
                Locked = 0 
	WHERE  SessionId  = id
        AND LockCookie  = lockCookie; 

END;
/



create or replace PROCEDURE TempRemoveStateItem
(
	id                       IN      	nvarchar2 DEFAULT NULL,
	lockCookie               IN      	int DEFAULT NULL
)
AS
BEGIN 
	
	DELETE  ASPStateTempSessions   
	WHERE  SessionId  = id
	AND LockCookie  = lockCookie; 

END;
/


create or replace PROCEDURE TempResetTimeout
(
	id                       IN      	nvarchar2 DEFAULT NULL
)
AS
        now timestamp;
BEGIN 

        now  :=  SYS_EXTRACT_UTC(SYSTIMESTAMP); 
	
	UPDATE  ASPStateTempSessions   
	SET	Expires = now + (1/24/60 * ASPStateTempSessions.Timeout) 
	WHERE  SessionId  = id; 
        
END;
/


create or replace PROCEDURE TempUpdateStateItem
(
	id                       IN      	nvarchar2 DEFAULT NULL,
	item                     IN      	blob DEFAULT NULL,
	timeout_                 IN      	int DEFAULT NULL,
	lockCookie               IN      	int DEFAULT NULL
)
AS
        expireDate                 timestamp;
BEGIN 

	expireDate := SYS_EXTRACT_UTC(SYSTIMESTAMP) + 1/24/60 * timeout_; -- add timeout minutes to the date
        
	UPDATE ASPStateTempSessions   
	SET Expires = expireDate,	
	    SessionItem = item,	
	    Timeout = timeout_,	
	    Locked = 0 
	WHERE  SessionId = id AND LockCookie = lockCookie; 

END;
/


create or replace PROCEDURE TempCreateOrUpdateStateItem
(
	id                       IN      	nvarchar2 DEFAULT NULL,
	item                     IN      	blob DEFAULT NULL,
	timeout_                 IN      	int DEFAULT NULL,
	lockCookie               IN      	int DEFAULT NULL
)
AS
	expireDate	timestamp;
	i			number;
	now			timestamp;
	nowLocal	timestamp;

BEGIN

	expireDate := SYS_EXTRACT_UTC(SYSTIMESTAMP) + 1/24/60 * timeout_; -- add timeout minutes to the date
	
	UPDATE ASPStateTempSessions
	SET Expires = expireDate, SessionItem = item, Timeout = timeout_, Locked = 0
	WHERE  SessionId = id AND LockCookie = lockCookie;

	i := sql%rowcount;

	IF i = 0 THEN
		now := SYS_EXTRACT_UTC(SYSTIMESTAMP);
		nowLocal := (SYSDATE);
		expireDate := now + 1/24/60 * timeout_; -- add timeout minutes to the date

		INSERT INTO ASPStateTempSessions(SessionId, SessionItem, timeout, Expires, Locked, LockDate, LockDateLocal, LockCookie)  
		VALUES (id, item, timeout_, expireDate, 0, now, nowLocal, 1); 
	END IF;
END;
/


create or replace PROCEDURE SMSUnlockSessions
AS
	now                        timestamp;
BEGIN  	
	now  :=  SYS_EXTRACT_UTC(SYSTIMESTAMP);
	
	UPDATE ASPStateTempSessions SET Locked = 0 
    WHERE SUBSTR(SessionId, 10, 3) = '000' AND Locked = 1 AND LockDate + 1/24/60 * 3 < SMSUnlockSessions.now; 

END;
/

