#!/bin/bash

source /opt/outsystems/platform/common.sh
checkIsRoot "serviceconfigurator must be run as root"

CONTENT=""

function printUsage() {
	echo "Usage: $0 [ -all | -controller | -frontend ]"
}

function enableAll() {
	CONTENT="$CONTENT\nRMI=\"ENABLED\""
	CONTENT="$CONTENT\nCONTROLLER=\"ENABLED\""
	CONTENT="$CONTENT\nDEPLOYER=\"ENABLED\""
	CONTENT="$CONTENT\nSCHEDULER=\"ENABLED\""
	CONTENT="$CONTENT\nLOGSERVER=\"ENABLED\""
}

function setController() {
	CONTENT="$CONTENT\nRMI=\"ENABLED\""
	RUN=$(/etc/init.d/outsystems status SCHEDULER | grep -o RUNNING)
	if [ "$RUN" == "RUNNING" ]; then
		NAME=$(printableName SCHEDULER)
		/etc/init.d/outsystems stop SCHEDULER
	fi
	RUN=$(/etc/init.d/outsystems status DEPLOYER | grep -o RUNNING)
	if [ "$RUN" == "RUNNING" ]; then
		NAME=$(printableName DEPLOYER)
		/etc/init.d/outsystems stop DEPLOYER
	fi
	CONTENT="$CONTENT\nCONTROLLER=\"ENABLED\""
	CONTENT="$CONTENT\nDEPLOYER=\"DISABLED\""
	CONTENT="$CONTENT\nSCHEDULER=\"DISABLED\""
	CONTENT="$CONTENT\nLOGSERVER=\"ENABLED\""
}

function setFrontEnd() {
	CONTENT="$CONTENT\nRMI=\"ENABLED\""
	RUN=$(/etc/init.d/outsystems status CONTROLLER | grep -o RUNNING)
	if [ "$RUN" == "RUNNING" ]; then
		NAME=$(printableName CONTROLLER)
		/etc/init.d/outsystems stop CONTROLLER
	fi
	CONTENT="$CONTENT\nCONTROLLER=\"DISABLED\""
	CONTENT="$CONTENT\nDEPLOYER=\"ENABLED\""
	CONTENT="$CONTENT\nSCHEDULER=\"ENABLED\""
	CONTENT="$CONTENT\nLOGSERVER=\"ENABLED\""
}

function setInteractive() {
	ENABLE=""
	ANSWER=""
	ask "RMI"
	CONTENT="$CONTENT\nRMI=\"$ENABLE\""
	ask "CONTROLLER"
	CONTENT="$CONTENT\nCONTROLLER=\"$ENABLE\""
	ask "DEPLOYER"
	CONTENT="$CONTENT\nDEPLOYER=\"$ENABLE\""
	ask "SCHEDULER"
	CONTENT="$CONTENT\nSCHEDULER=\"$ENABLE\""
	ask "LOGSERVER"
	CONTENT="$CONTENT\nLOGSERVER=\"$ENABLE\""
}

function ask() {
	NAME=$(printableName $1)
	echo "Enable $NAME?"
	read ANSWER
	if [ "$ANSWER" == "y" -o "$ANSWER" == "yes" -o "$ANSWER" == "Y" -o "$ANSWER" == "YES" ]; then
		ENABLE="ENABLED"
	else
		ENABLE="DISABLED"
		RUN=$(/etc/init.d/outsystems status $1 | grep -o RUNNING)
		if [ "$RUN" == "RUNNING" ]; then
			/etc/init.d/outsystems stop $1
		fi
	fi
	echo -e "\t$NAME - $ENABLE"
}

if [ -z "$1" ]; then
	printUsage
	exit 1
fi

case "$1" in
-all)
	enableAll
	;;
-controller)
	setController
	;;
-frontend)
	setFrontEnd
	;;
-interactive)
	setInteractive
	;;
*)
	printUsage
	exit 1
	;;
esac

echo -e $CONTENT > $SERVICESCONFIG
chown root:outsystems $SERVICESCONFIG
chmod 640 $SERVICESCONFIG
